﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Telerik.WinControls.RadControlSpy;
using Telerik.WinControls.UI;

namespace WindowsFormsApplication37
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            this.InitializeComponent();

            this.Controls.Add(new MyScheduler() { Dock = DockStyle.Fill });

            new RadControlSpyForm().Show();
        }
    }

    public class MyScheduler : RadScheduler
    {
        DragFeedbackElement dragFeedBack;
        IEvent resizedAppointment;
        TimeSpan resizeOffset;

        bool isDragging;
        DateTime targetDate;
        bool isResizing;
        bool isResizingDown;

        protected override void OnMouseDown(MouseEventArgs e)
        {
            AppointmentElement element = this.ElementTree.GetElementAtPoint(e.Location) as AppointmentElement;
            if (element != null && element.Appointment != null && element.Appointment.MasterEvent != null)
            {
                if (this.IsInResizeLocation(e.Location, element))
                {
                    this.BeginResize(element, e);
                }
                else
                {
                    this.BeginDrag(element);
                }
            }
            else
            {
                base.OnMouseDown(e);
            }
        }
  
        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (this.isResizing)
            {
                ProcessResize(e);
            }
            else if (this.isDragging)
            {
                ProcessDrag(e);
            }
            else
            {
                foreach (AppointmentElement element in SchedulerUIHelper.GetAppointmentElements(this))
                {
                    if (this.IsInResizeLocation(e.Location, element))
                    {
                        this.Cursor = Cursors.SizeNS;
                        break;
                    }
                    else
                    {
                        this.Cursor = Cursors.Default;
                    }
                }

                base.OnMouseMove(e);
            }
        }
  
        protected override void OnMouseUp(MouseEventArgs e)
        {
            if (this.isResizing)
            {
                EndResize();
            }
            else if (this.isDragging)
            { 
                EndDrag();
            }
            else
            {
                base.OnMouseUp(e);
            }
        }

        private void BeginDrag(AppointmentElement element)
        {
            this.Capture = true;
            this.isDragging = true;

            this.dragFeedBack = new DragFeedbackElement(this, this.ActiveView, element.Appointment);
            this.dragFeedBack.BackColor = Color.White;
            this.dragFeedBack.Opacity = 0.5;
            this.dragFeedBack.MinSize = element.Size;
            this.dragFeedBack.StretchHorizontally = this.dragFeedBack.StretchVertically = false;
            this.dragFeedBack.PositionOffset = new SizeF(element.ControlBoundingRectangle.X, element.ControlBoundingRectangle.Y);

            element.Visibility = Telerik.WinControls.ElementVisibility.Hidden;
            this.targetDate = element.Start;

            this.RootElement.Children.Add(this.dragFeedBack);
        }

        private void BeginResize(AppointmentElement element, MouseEventArgs e)
        {
            this.Capture = true;
            this.isResizing = true;
            this.isResizingDown = element.ControlBoundingRectangle.Bottom - e.Y < e.Y - element.ControlBoundingRectangle.Top;
            this.resizedAppointment = element.Appointment.MasterEvent;

            this.resizeOffset = (this.isResizingDown) ?
                                                       element.Appointment.MasterEvent.End - element.Appointment.End :
                                                     element.Appointment.MasterEvent.Start - element.Appointment.Start;
        }

        private void ProcessDrag(MouseEventArgs e)
        {
            foreach (SchedulerCellElement cell in SchedulerUIHelper.GetCells(this))
            {
                if (cell.ControlBoundingRectangle.Contains(e.Location))
                {
                    this.dragFeedBack.PositionOffset = new SizeF(cell.ControlBoundingRectangle.X, cell.ControlBoundingRectangle.Y);
                    this.targetDate = cell.Date;
                    this.Invalidate();
                }
            }
        }

        private void ProcessResize(MouseEventArgs e)
        {
            foreach (SchedulerCellElement cell in SchedulerUIHelper.GetCells(this))
            {
                if (cell.ControlBoundingRectangle.Contains(e.Location))
                {
                    TimeSpan cellDuration = TimeSpan.FromHours(1);

                    SchedulerDayViewBase dayView = this.ActiveView as SchedulerDayViewBase;
                    if (dayView != null)
                    {
                        cellDuration = TimeSpan.FromMinutes((int)dayView.RangeFactor);
                    }

                    if (this.isResizingDown)
                    {
                        DateTime newDate = cell.Date.Add(this.resizeOffset) > this.resizedAppointment.Start ?
                            cell.Date.Add(this.resizeOffset) : this.resizedAppointment.Start;

                        this.resizedAppointment.End =
                            newDate.Add(cellDuration);
                    }
                    else
                    {
                        DateTime newDate = cell.Date.Add(this.resizeOffset) < this.resizedAppointment.End.Add(-cellDuration) ?
                            cell.Date.Add(this.resizeOffset) : this.resizedAppointment.End.Add(-cellDuration);

                        this.resizedAppointment.Start = newDate;
                    }
                }
            }
        }

        private void EndDrag()
        {
            TimeSpan duration = this.dragFeedBack.Appointment.MasterEvent.Duration;
            TimeSpan offset = this.dragFeedBack.Appointment.MasterEvent.Start - this.dragFeedBack.Appointment.Start;
            
            DateTime newDate = this.targetDate.Add(offset);
            
            if (this.dragFeedBack.Appointment.MasterEvent.Start != newDate)
            {
                this.dragFeedBack.Appointment.MasterEvent.Start = this.targetDate.Add(offset);
                this.dragFeedBack.Appointment.MasterEvent.Duration = duration;
            }
            else
            {
                foreach (AppointmentElement element in SchedulerUIHelper.GetAppointmentElements(this))
                {
                    element.Visibility = Telerik.WinControls.ElementVisibility.Visible;
                }
            }

            this.Capture = false;
            this.isDragging = false;
            this.RootElement.Children.Remove(this.dragFeedBack);
            this.dragFeedBack.Dispose();
            this.dragFeedBack = null;
            this.Invalidate();
        }
  
        private void EndResize()
        {
            this.isResizing = false;
            this.Capture = false;
            this.Cursor = Cursors.Default;
            this.resizedAppointment = null;
        }

        private bool IsInResizeLocation(Point point, AppointmentElement element)
        {
            RectangleF rect = element.ControlBoundingRectangle;
            if (rect.Left < point.X && point.X < rect.Right)
            {
                if ((point.Y > rect.Top && point.Y - rect.Top < 5) ||
                    (point.Y < rect.Bottom && rect.Bottom - point.Y < 5))
                {
                    return true;
                }
            }

            return false;
        }

        public override string ThemeClassName
        {
            get
            {
                return typeof(RadScheduler).FullName;
            }
            set
            {
                base.ThemeClassName = value;
            }
        }
    }
}
